;nyquist plug-in
;version 4
;type process
;name "Dynamic Mirror..."
;action "Processing ..."
;author "Steve Daulton"
;copyright "Released under terms of the GNU General Public License version 2"

;control mode "Follow peak or RMS level" choice "Peak,RMS" 0
;control inverse "Follow first track as" choice "Mirror,Inverse (ducking)" 0
;control resolution "Time resolution" float "seconds" 0.5 0.01 10
;control range-min "Control range min" float "" 0 0 1
;control range-max "Control range max" float "" 1 0 1

(defun mono (sig)
  "Convert to mono if necessary by averaging channels"
  (setf sig (multichan-expand #'apply-range sig))
  (if (arrayp sig)
      (mult 0.5 (sum (aref sig 0)(aref sig 1)))
      sig))

(defun apply-range (sig)
  "Trim and scale the control track audio to range"
  (let* ((range (- range-max range-min))
         (sig (s-abs sig))
         (sig (diff sig range-min))
         (sig (mult sig (/ range))))
    (s-max 0 (clip sig 1))))

(defun is-last-track()
  "Is this the final selected track? Returns bool"
  (= (get '*track* 'index)
     (length (get '*selection* 'tracks))))

(defun create-envelope()
  "Generate envelope and put it into the *scratch* "
  (let ((step (truncate (* resolution *sound-srate*)))
        (op (- 2 mode)))
    (setf *scratch*
      (snd-copy (snd-avg (mono *track*) step step op)))))

(defun apply-envelope()
  "Apply envelope to track"
  (let ((env *scratch*)
        (offset (* resolution (/ (1+ mode) 2.0)))
        (initial-amp (snd-fetch (snd-copy *scratch*))))
    ; release *scratch* when we're finished
    (when (is-last-track) (setf *scratch* '*unbound*))
    ; invert envelope if requested
    (cond
      ((= inverse 0) ; Inverse = false
        (setf T0-amp 0))
      (t
        (setf T0-amp 1)
        (setf initial-amp (- 1 initial-amp))
        (setf env (diff 1 env))))
    (mult *track*
      (sim
        ; ramp to initial amplitude
        (abs-env (pwlv  T0-amp offset initial-amp))
        (at-abs offset (cue env))))))


(cond
  ((> (get-duration 1) 1800)
    (format nil "Error.~%~%Selection too long.~%~
                 Reduce the selection to 30 mins maximum"))
  ((< (get-duration 0.5) resolution)
    (format nil "Error.~%~%The 'Time Resolution' should be considerably~%~
                 shorter than the length of the selection.~%~
                 The absolute maximum allowed 'Time Resolution' is~%~
                 half of the selection length."))
  ((>= range-min range-max)
    (format nil "Error.~%~%\'Control range min\' must be less than \'Control range max\'."))
  ((= (get '*track* 'index) 1)
    (create-envelope)
    (if (is-last-track) ; only 1 track selected
        (apply-envelope)
        ; return *track* to prevent audio from being released from *scratch*.
        *track*))
  (T (apply-envelope)))
